﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

/* Approximation of a given function by Chebychev polynomials*/
/* www.mosismath.com                                         */

namespace WindowsFormsApplication1
{

    public partial class MainWin : Form
    {
        public const int order =1000; // number of reference points
        public const int datapoints = 200; // number of datapoints for the interpolation
      
        double a = -1;
        double b = 1;
        public double deltaX;

        double[] yp = new double[datapoints+1];
        double[] xp = new double[datapoints+1];
        double[] y_k = new double[order+1];
        double[] x_k = new double[order+1];
        public double[] c = new double[order + 1];


        private double f_x(double x)
        {
           return (x*x*x); 
        }
        
        public MainWin()
        {
            int k;
            InitializeComponent();
        }

        public void CalcCParams() // calculation of the Fourier components  
        {                       
            int k, n;
            if (order > 0)
            {
                for (k = 0; k < order; k++)
                {
                    c[k] = 0;
                    for (n = 0; n < (order); n++)
                    {
                        c[k] = c[k] + Math.Cos(2.0 * Math.PI * (k * n) / (double)order) * y_k[n];
                    }
                    c[k] = c[k] / order * 2;
                }
                c[0] = c[0] / 2.0;
            }
        }

        public void Approximate(int maxPoints, double maxTime)
        {
            int i;
            for (i = 0; i <= maxPoints; i++)
            {
                yp[i] = 0;
                xp[i] = (a + (b - a) * (double)(i) / (double)maxPoints);
                yp[i] = CalcChebychev(xp[i], 6);
            }
        }

        public void DrawGraph(int maxPoints, double minX, double maxX, bool doClear)
        {
            Point p1, p2;
            int j;
            double maxTime = -1.0;
            double maxVal = -1.0;
            double minVal = 1.0;
            double scalefactor;
            bool drawCeroline;
            for (j = 0; j < maxPoints - 1; j++)
            {
                if ((xp[j] >= minX) && (xp[j] <= maxX))
                {
                    if (maxTime < xp[j])
                        maxTime = xp[j];
                    if (maxVal < yp[j])
                        maxVal = yp[j];
                    if (minVal > yp[j])
                        minVal = yp[j];
                }
            }
            maxTime = (maxX - minX) * 1.05;
            if ((maxTime > 0) && (maxVal > 0))
            {
                if (minVal < 0)
                {
                        if (maxVal > Math.Abs(minVal))
                            scalefactor = pGraph.Height / maxVal / 2.2;
                        else
                            scalefactor = pGraph.Height / Math.Abs(minVal) / 2.2;
                    drawCeroline = true;
                }
                else
                {
                    scalefactor = pGraph.Height / (maxVal) / 1.1;
                    drawCeroline = false;
                }
                p1 = new Point();
                p2 = new Point();
                Graphics g = pGraph.CreateGraphics();
                if (doClear)
                    g.Clear(Color.White);
                Pen bluePen = new Pen(Color.Blue, 2);
                Pen redPen = new Pen(Color.Red, 2);
                Pen blackPen = new Pen(Color.Black, 2);
                if (drawCeroline)
                {
                    p1.X = 0;
                    p1.Y = (pGraph.Height / 2);
                    p2.X = pGraph.Width;
                    p2.Y = (pGraph.Height / 2);
                    g.DrawLine(blackPen, p1, p2);
                }
                bluePen.Width = 1;
                if (maxTime > 0) // draw interpolated graph
                {
                    for (j = 0; j < maxPoints - 1; j++)
                    {
                        if ((xp[j] >= minX) && (xp[j] <= maxX))
                        {
                            p1.X = Convert.ToInt32((xp[j] - minX) * pGraph.Width / (double)maxTime);
                            p2.X = Convert.ToInt32((xp[j + 1] - minX) * pGraph.Width / (double)maxTime);
                            if (drawCeroline)
                            {
                                p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                                p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                            }
                            else
                            {
                                p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j] * scalefactor));
                                p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(yp[j + 1] * scalefactor));
                            }
                            if (p2.X > p1.X)
                                g.DrawLine(bluePen, p1, p2);
                        }
                    }

                    for (j = 2; j < order; j+=10) // mark reference points
                    {
                        p1.X = Convert.ToInt32((x_k[j-2] - minX) * pGraph.Width / (double)maxTime);
                        p2.X = Convert.ToInt32((x_k[j] - minX) * pGraph.Width / (double)maxTime);
                        if (drawCeroline)
                        {
                            p1.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j - 2] * scalefactor));
                            p2.Y = (pGraph.Height / 2) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        }
                        else
                        {
                            p1.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j-2] * scalefactor));
                            p2.Y = (pGraph.Height) - Convert.ToInt32(Math.Round(y_k[j] * scalefactor));
                        }
                        g.DrawLine(redPen, p1, p2);
                    }
                }
                g.Dispose();
            }
        }
         

        public double CalcChebychev(double x, int order)
        {
            int i;
            double[] t = new double[7];
            double value;
            t[0] = 1;
            t[1] = x;
            for (i = 2; i < order; i++)
                t[i] = 2 * x * t[i - 1] - t[i - 2];
            value = c[0];
            for (i = 1; i < order; i++)
                value = value + c[i] * t[i]; ;
            return value;
        }


        private void button1_Click(object sender, EventArgs e)
        {
            int i;
            deltaX = (b - a) / (double)order;

            for(i = 0 ; i < order; i++)
            {
                x_k[i] = (double)(i) / (double)order;
                y_k[i] = f_x(Math.Cos(x_k[i] * Math.PI * 2.0 ));
            }
            CalcCParams();
            // initialize supporting points
            i = 0;
            for (double x = a; x <= b; x += deltaX)
            {
                x_k[i] = x;
                y_k[i] = f_x(x);
                i++;
            }
            Approximate(datapoints, b);
            DrawGraph(datapoints, a, b, true);
        }

        private void pGraph_Paint(object sender, PaintEventArgs e)
        {
            DrawGraph(datapoints, a, b, false);
        }  
    }
}
